# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['migrado']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0',
 'docker[tls]>=4.0,<5.0',
 'python-arango>=4.4,<5.0',
 'pyyaml>=3.13']

entry_points = \
{'console_scripts': ['migrado = migrado:migrado']}

setup_kwargs = {
    'name': 'migrado',
    'version': '0.1.1',
    'description': 'ArangoDB migrations and batch processing manager',
    'long_description': 'migrado\n=======\n\n[![PyPI package](https://badge.fury.io/py/migrado.svg)](https://pypi.org/project/migrado/)\n[![Build status](https://travis-ci.org/protojour/migrado.svg?branch=master)](https://travis-ci.org/protojour/migrado)\n\n🥑 ArangoDB migrations and batch processing manager.\n\nmigrado is a command-line client that can help build and run schema or data migrations against your ArangoDB instance. \n\nmigrado utilizes ArangoDB Transactions when running data migrations to ensure failed scripts are rolled back automatically. [Docker](https://docs.docker.com/install/) is required to run schema migrations, however no transaction safety is available at this point.\n\n**migrado should be considered alpha software.** Make sure you test well before using in a production setting.\n\nIf you have trouble, open an issue. Contributions are welcome.\n\nInstallation\n------------\n\n```bash\n$ pip install --user migrado\n```\n\nUsage\n-----\n\nmigrado can create a migrations directory and generate an initial set of collections from the given schema file:\n\n```bash\n$ migrado init --schema schema.yml\n```\n\nSee [YAML schemas](#yaml-schemas) for details. If no schema is specified, migrado will create an empty initial migration.\n\nTo make a new template migration script:\n\n```bash\n$ migrado make --name rewrite_names\n```\n\nThis will create a new file, `migrations/0002_rewrite_names.js` (`--name` is optional), which you can edit as you see fit. See [Migration scripts](#migration-scripts) for details.\n\nWhen you are ready, run all migrations not currently ran against the database: \n\n```bash\n$ migrado run\n```\n\nmigrado stores migration state in a configurable collection, see `--help` or [Environment vars](#environment-vars) for details.\n\nIf you wrote a `reverse()` migration, you can revert to an earlier point by specifying a target migration id. To revert to the initial migration:\n\n```bash\n$ migrado run --target 0001\n```\n\nUse the `--help` option for help on any command when using the client.\n\nEnvironment vars\n----------------\n\nThe following environment variables are employed by migrado:\n\n- `MIGRADO_PATH`: Specifies the path to the migrations directory, replaces `-p`, `--path` (default: `migrations`).\n- `MIGRADO_DB`: Specifies the ArangoDB database name for generated migrations to interact with, replaces `-d`, `--db` (no default, but required for the `run` command).\n- `MIGRADO_COLL`: Specifies ArangoDb collection name to store migration state in, replaces `-c`, `--state-coll` (default: `migrado`).\n- `MIGRADO_HOST`: Specifies the database host for running migrations, replaces `-H`, `--host` (default: `localhost`).\n- `MIGRADO_PORT`: Specifies the database port for running migrations, replaces `-P`, `--port` (default: `8529`).\n- `MIGRADO_DOCKER_IMAGE`: Specifies the Docker image (and optionally tag) for the container running migrations, replaces `-I`, `--docker-image` (default: `arangodb`).\n- `MIGRADO_DOCKER_NETWORK`: Specifies a Docker network mode or name for running migrations, replaces `-N`, `--docker-network`. Valid values are `host` (default, use the host network), or any network name.\n- `MIGRADO_DOCKER_SERVICE`: Specifies a Docker service to connect to when running migrations, replaces `-S`, `--docker-service` (default: same value as `MIGRADO_HOST`).\n\nIf your ArangoDB instance is itself running on Docker, `MIGRADO_DOCKER_NETWORK` and `MIGRADO_DOCKER_SERVICE` may be configured so the created container can connect easily to the correct network and service.\n\nFor connection to the Docker daemon, additional environment variables are accepted; `DOCKER_HOST`, `DOCKER_TLS_VERIFY` and `DOCKER_CERT_PATH`. They are same as those used by the Docker command-line client.\n\nYAML schemas\n------------\n\nArangoDB may be schemaless, but in a larger project it still makes sense to keep a schema spec up to date, both for an overview of collections and their data structures, and as a basis for validation.\n\nmigrado uses a schema model based on JSON Schema, in YAML, and can use this to generate an initial migration for the collections available in your database.\n\nExample schema:\n\n```yaml\n---\nall: &all\n  _id:\n    type: string\n    readOnly: true\n  _key:\n    type: string\n    readOnly: true\n  _rev:\n    type: string\n    readOnly: true\n\nedges: &edges\n  _from:\n    type: string\n  _to:\n    type: string\n\ncollections:\n\n  books:\n    type: object\n    properties:\n      <<: *all\n      title:\n        type: string\n      isbn:\n        type: string\n    required:\n      - title\n      - isbn\n\n  authors: \n    # Note, you do not actually need to specify the object schema,\n    # but they can be used in API specs (e.g. OpenAPI) and/or validation,\n    # and may be handled by migrado in the future.\n\nedge_collections:\n\n  # authors --> books \n  author_of:\n    type: object\n    properties:\n      <<: *all\n      <<: *edges\n    required:\n      - _from\n      - _to\n``` \n\nMigration scripts\n-----------------\n\nMigration scripts are structured so they may be parsed and run easily by both migrado and ArangoDB. In addition, they are structured so they may be run manually against ArangoDB using `arangosh`.\n\nThere are two types of script, **data** and **schema** migration scripts.\n\n### Data migrations\n\nYou need to declare all collections subject to write operations using the syntax `// write collection_name`, because ArangoDB needs this information for locking during transactions. We\'ve made the declaration explicit to reduce errors. _Attempting to write to collections not declared in this way will cause the migration to fail._\n\nIn general, a reverse migration should do the logical opposite of a forward migration. `forward()` and `reverse()` functions can contain anything that the ArangoDB V8 engine understands, but must be fully self-contained. _Anything outside these functions is ignored and unavailable when running migrations._\n\nHere\'s an example migration script for adding `new_field` in collection `things`:\n\n```javascript\n// write things\n\nfunction forward() {\n    var db = require("@arangodb").db\n    db._query(`\n        FOR thing IN things\n            UPDATE thing WITH { new_field: "some value" } IN things\n    `)\n}\n\nfunction reverse() {\n    var db = require("@arangodb").db\n    db._query(`\n        FOR thing IN things\n            REPLACE thing WITH UNSET(thing, "new_field") IN things\n    `)\n}\n```\n\nPlease make sure you read [limitations when running transactions](https://www.arangodb.com/docs/stable/transactions-limitations.html) in the ArangoDB documentation. In particular, _creation and deletion of databases, collections, and indexes_ is not allowed in transactions.\n\nIf a migration contains such operations, you will be asked if you want to run the migration through Docker.\n\n### Schema migrations\n\nSchema migrations are stuctured in the same way as data migrations, but are run against `arangosh` in a container from an [official arangodb Docker image](https://hub.docker.com/_/arangodb). There is no transaction safety when running schema migrations.\n\nSchema migrations are structured the same way as data migrations, but `// write` declarations are not required. All operations are allowed.\n\nHere\'s an example migration script generated from the YAML schema above:\n\n```javascript\nfunction forward() {\n    var db = require("@arangodb").db\n    db._createDocumentCollection("books")\n    db._createDocumentCollection("authors")\n    db._createEdgeCollection("author_of")\n}\n\nfunction reverse() {\n    var db = require("@arangodb").db\n    db._drop("books")\n    db._drop("authors")\n    db._drop("author_of")\n}\n```\n\nPlease be careful when running schema migrations in reverse. As you can see, the `reverse()` function above would drop your collections if you were to reverse beyond this point. Currently, you will not be able to do so for an initial migration.\n\nTODO\n----\n- [ ] Authentication against Docker and ArangoDB\n- [ ] Transaction-like safe runs for schema migrations\n- [ ] Automatic diffing of schema migrations\n- [ ] Using migrado to add indexes\n- [ ] Using migrado to add recurring tasks\n\nLicense\n-------\n\nmigrado is copyright © 2019 Protojour AS, and is licensed under MIT. See [LICENSE.txt](https://github.com/protojour/migrado/blob/master/LICENSE.txt) for details.\n',
    'author': 'Eirik Krogstad',
    'author_email': 'eirikkr@gmail.com',
    'url': 'https://github.com/protojour/migrado',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
